#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-01-01
#
# Monitor status of OSPFv3 areas
#
# 2021-09-17: rewritten for CMK 2.0
#
# sample snmpwalk
# .1.3.6.1.2.1.191.1.2.1.2.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.2.1.2.3 = INTEGER: 1
# .1.3.6.1.2.1.191.1.2.1.3.0 = Counter32: 44
# .1.3.6.1.2.1.191.1.2.1.3.3 = Counter32: 2
# .1.3.6.1.2.1.191.1.2.1.4.0 = Gauge32: 4
# .1.3.6.1.2.1.191.1.2.1.4.3 = Gauge32: 1
# .1.3.6.1.2.1.191.1.2.1.5.0 = Gauge32: 0
# .1.3.6.1.2.1.191.1.2.1.5.3 = Gauge32: 0
# .1.3.6.1.2.1.191.1.2.1.6.0 = Gauge32: 16
# .1.3.6.1.2.1.191.1.2.1.6.3 = Gauge32: 7
# .1.3.6.1.2.1.191.1.2.1.7.0 = Gauge32: 534138
# .1.3.6.1.2.1.191.1.2.1.7.3 = Gauge32: 352980
# .1.3.6.1.2.1.191.1.2.1.8.0 = INTEGER: 2
# .1.3.6.1.2.1.191.1.2.1.8.3 = INTEGER: 2
# .1.3.6.1.2.1.191.1.2.1.9.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.2.1.9.3 = INTEGER: 1
# .1.3.6.1.2.1.191.1.2.1.10.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.2.1.10.3 = INTEGER: 1
# .1.3.6.1.2.1.191.1.2.1.11.0 = INTEGER: 2
# .1.3.6.1.2.1.191.1.2.1.11.3 = INTEGER: 2
# .1.3.6.1.2.1.191.1.2.1.12.0 = INTEGER: 3
# .1.3.6.1.2.1.191.1.2.1.12.3 = INTEGER: 3
# .1.3.6.1.2.1.191.1.2.1.13.0 = Gauge32: 0
# .1.3.6.1.2.1.191.1.2.1.13.3 = Gauge32: 0
# .1.3.6.1.2.1.191.1.2.1.14.0 = Counter32: 0
# .1.3.6.1.2.1.191.1.2.1.14.3 = Counter32: 0
# .1.3.6.1.2.1.191.1.2.1.15.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.2.1.15.3 = INTEGER: 1
# .1.3.6.1.2.1.191.1.2.1.16.0 = INTEGER: 2
# .1.3.6.1.2.1.191.1.2.1.16.3 = INTEGER: 2
#
# sample string_table
# [
#  ['0', '1', '17', '4', '0', '18', '646544', '2', '1', '2', '3', '0', '0', '1', '2'],
#  ['1', '1', '3', '1', '0', '7', '317410', '2', '1', '2', '3', '0', '0', '1', '2']
# ]
#
from dataclasses import dataclass
from typing import Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    State,
    SNMPTree,
    exists,
    Metric,
    OIDEnd,
    check_levels,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.utils.ospfv3 import (
    ospf_area_impotasextern,
    ospf_area_summary,
    ospf_area_translatorrole,
    ospf_area_translatorrolestate,
    ospf_area_submetrictype,
    ospf_area_teeenabled,
    get_area_type
)


@dataclass
class OspfV3Area:
    AreaImportAsExtern: str
    AreaSummary: str
    AreaSpfRuns: int
    AreaBdrRtrCount: int
    AreaAsBdrRtrCount: int
    AreaScopeLsaCount: int
    AreaScopeLsaCksumSum: int
    AreaStubMetric: int
    AreaNssaTranslatorRole: str
    AreaNssaTranslatorState: str
    AreaNssaTranslatorStabInterval: int
    AreaNssaTranslatorEvents: int
    AreaStubMetricType: str
    AreaTEEnabled: str
    AreaType: str


def parse_ospfv3_area(string_table: StringTable) -> Dict[str, OspfV3Area]:
    areas = {}
    for area in string_table:
        AreaId, AreaImportAsExtern, AreaSpfRuns, AreaBdrRtrCount, AreaAsBdrRtrCount, AreaScopeLsaCount, \
        AreaScopeLsaCksumSum, AreaSummary, AreaStubMetric, AreaNssaTranslatorRole, AreaNssaTranslatorState, \
        AreaNssaTranslatorStabInterval, AreaNssaTranslatorEvents, AreaStubMetricType, AreaTEEnabled, = area

        areas[AreaId] = OspfV3Area(
            AreaImportAsExtern=ospf_area_impotasextern(AreaImportAsExtern),
            AreaSummary=ospf_area_summary(AreaSummary),
            AreaSpfRuns=int(AreaSpfRuns),
            AreaBdrRtrCount=int(AreaBdrRtrCount),
            AreaAsBdrRtrCount=int(AreaAsBdrRtrCount),
            AreaScopeLsaCount=int(AreaScopeLsaCount),
            AreaScopeLsaCksumSum=int(AreaScopeLsaCksumSum),
            AreaStubMetric=int(AreaStubMetric),
            AreaNssaTranslatorRole=ospf_area_translatorrole(AreaNssaTranslatorRole),
            AreaNssaTranslatorState=ospf_area_translatorrolestate(AreaNssaTranslatorState),
            AreaNssaTranslatorStabInterval=int(AreaNssaTranslatorStabInterval),
            AreaNssaTranslatorEvents=int(AreaNssaTranslatorEvents),
            AreaStubMetricType=ospf_area_submetrictype(AreaStubMetricType),
            AreaTEEnabled=ospf_area_teeenabled(AreaTEEnabled),
            AreaType=get_area_type(AreaImportAsExtern, AreaSummary)
        )
    return areas


def discovery_ospfv3_area(section: Dict[str, OspfV3Area]) -> DiscoveryResult:
    for area in section.keys():
        yield Service(item=area)


def check_ospfv3_area(item, params, section: Dict[str, OspfV3Area]) -> CheckResult:
    try:
        area = section[item]
    except KeyError:
        yield Result(state=State.UNKNOWN, notice='Item not found in SNMP DATA')
        return

    for metric, value, label in [
        ('spfruns', area.AreaSpfRuns, 'SPF runs'),
        ('scopelsacount', area.AreaScopeLsaCount, 'LSAs'),
        ('bdrrtcount', area.AreaBdrRtrCount, 'ABRs'),
        ('asbdrrtrcount', area.AreaAsBdrRtrCount, 'ASBRs'),

    ]:
        yield from check_levels(
            value=value,
            label=label,
            metric_name=f'ospfv3_area_{metric}',
            render_func=lambda v: f'{v:.0f}'
        )
    yield Result(state=State.OK, notice=f'Area type: {area.AreaType}')
    yield Result(state=State.OK, notice=f'Area scope LSA checksum: {area.AreaScopeLsaCksumSum}')
    yield Result(state=State.OK, notice=f'Area import as extern: {area.AreaImportAsExtern}')
    yield Result(state=State.OK, notice=f'Area summary: {area.AreaSummary}')
    yield Result(state=State.OK, notice=f'Area stub metric: {area.AreaStubMetric}')
    yield Result(state=State.OK, notice=f'Area NSSA translator role: {area.AreaNssaTranslatorRole}')
    yield Result(state=State.OK, notice=f'Area NSSA translator state: {area.AreaNssaTranslatorState}')
    yield Result(state=State.OK, notice=f'Area NSSA trabslator stub interval: {area.AreaNssaTranslatorStabInterval}')
    yield Result(state=State.OK, notice=f'Area NSSA stub metric type: {area.AreaStubMetricType}')
    yield Result(state=State.OK, notice=f'Area Traffic engineering  {area.AreaTEEnabled}')

    yield Metric(name='ospfv3_area_nssatranslatorevents', value=area.AreaNssaTranslatorEvents)


register.snmp_section(
    name='ospfv3_area',
    parse_function=parse_ospfv3_area,
    fetch=SNMPTree(
        base='.1.3.6.1.2.1.191.1.2.1',  # OSPFV3-MIB::ospfv3AreaEntry
        oids=[
            OIDEnd(),  # ospfv3AreaId
            '2',  # ospfv3AreaImportAsExtern
            '3',  # ospfv3AreaSpfRuns
            '4',  # ospfv3AreaBdrRtrCount
            '5',  # ospfv3AreaAsBdrRtrCount
            '6',  # ospfv3AreaScopeLsaCount
            '7',  # ospfv3AreaScopeLsaCksumSum
            '8',  # ospfv3AreaSummary
            '10',  # ospfv3AreaStubMetric
            '11',  # ospfv3AreaNssaTranslatorRole
            '12',  # ospfv3AreaNssaTranslatorState
            '13',  # ospfv3AreaNssaTranslatorStabInterval
            '14',  # ospfv3AreaNssaTranslatorEvents
            '15',  # ospfv3AreaStubMetricType
            '16',  # ospfv3AreaTEEnabled
        ]
    ),
    detect=exists('.1.3.6.1.2.1.191.1.2.1.2.*')  # OSPFV3-MIB::ospfv3AreaImportAsExtern
)

register.check_plugin(
    name='ospfv3_area',
    service_name='OSPFv3 area %s',
    discovery_function=discovery_ospfv3_area,
    check_function=check_ospfv3_area,
    check_default_parameters={
    },
    check_ruleset_name='ospfv3_area',
)
