#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-01-01
#
# Monitor status of OSPFv3 general status
#
# 2021-09-17: rewritten for CMK 2.0
# 2021-10-02: added WATO options
#
# sample snmpwalk
# .1.3.6.1.2.1.191.1.1.1.0 = Gauge32: 50529054
# .1.3.6.1.2.1.191.1.1.2.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.1.3.0 = INTEGER: 3
# .1.3.6.1.2.1.191.1.1.4.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.1.5.0 = INTEGER: 2
# .1.3.6.1.2.1.191.1.1.6.0 = Gauge32: 0
# .1.3.6.1.2.1.191.1.1.7.0 = Gauge32: 0
# .1.3.6.1.2.1.191.1.1.8.0 = Counter32: 313
# .1.3.6.1.2.1.191.1.1.9.0 = Counter32: 63
# .1.3.6.1.2.1.191.1.1.10.0 = Gauge32: 0
# .1.3.6.1.2.1.191.1.1.11.0 = INTEGER: -1
# .1.3.6.1.2.1.191.1.1.12.0 = Gauge32: 0
# .1.3.6.1.2.1.191.1.1.13.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.1.14.0 = Gauge32: 100000
# .1.3.6.1.2.1.191.1.1.15.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.1.16.0 = Gauge32: 120
# .1.3.6.1.2.1.191.1.1.17.0 = INTEGER: 2
# .1.3.6.1.2.1.191.1.1.18.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.1.19.0 = Gauge32: 0
# .1.3.6.1.2.1.191.1.1.20.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.1.21.0 = INTEGER: 2
# .1.3.6.1.2.1.191.1.1.22.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.1.23.0 = INTEGER: 1
# .1.3.6.1.2.1.191.1.1.24.0 = 0
# .1.3.6.1.2.1.191.1.1.25.0 = 0
#
# sample string_table
# [['0', '16843038', '1', '3', '1', '2', '0', '0', '256', '51', '0', '100000', '1', '1']]

from dataclasses import dataclass
from typing import Optional

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    State,
    SNMPTree,
    exists,
    Metric,
    OIDEnd,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.utils.ospfv3 import (
    ospf_stub_router_advertisement,
    ospf_admin_status,
    ospf_abr_asbr_status,
    ospf_stub_router_support,
    render_ipv4_neighbor_id,
)


@dataclass
class OspfV3General:
    RouterId: str
    AdminStatus: str
    VersionNumber: int
    AreaBdrRtrStatus: str
    ASBdrRtrStatus: str
    AsScopeLsaCount: int
    AsScopeLsaCksumSum: int
    RxNewLsas: int
    ExtLsaCount: int
    ExtAreaLsdbLimit: int
    ReferenceBandwidth: int
    StubRouterSupport: str
    StubRouterAdvertisement: str


def parse_ospfv3_general(string_table: StringTable) -> Optional[OspfV3General]:
    try:
        OID_END, RouterId, AdminStatus, VersionNumber, AreaBdrRtrStatus, ASBdrRtrStatus, AsScopeLsaCount, \
        AsScopeLsaCksumSum, OriginateNewLsas, RxNewLsas, ExtLsaCount, ExtAreaLsdbLimit, \
        ReferenceBandwidth, StubRouterSupport, StubRouterAdvertisement = string_table[0]
    except (ValueError, IndexError):
        return

    return OspfV3General(
        RouterId=render_ipv4_neighbor_id(RouterId),
        AdminStatus=ospf_admin_status(AdminStatus),
        VersionNumber=int(VersionNumber),
        AreaBdrRtrStatus=ospf_abr_asbr_status(AreaBdrRtrStatus),
        ASBdrRtrStatus=ospf_abr_asbr_status(ASBdrRtrStatus),
        AsScopeLsaCount=int(AsScopeLsaCount),
        AsScopeLsaCksumSum=int(AsScopeLsaCksumSum),
        RxNewLsas=int(RxNewLsas),
        ExtLsaCount=int(ExtLsaCount),
        ExtAreaLsdbLimit=int(ExtAreaLsdbLimit),
        ReferenceBandwidth=int(ReferenceBandwidth),
        StubRouterSupport=ospf_stub_router_support(StubRouterSupport),
        StubRouterAdvertisement=ospf_stub_router_advertisement(StubRouterAdvertisement),
    )


def discovery_ospfv3_general(params, section: OspfV3General) -> DiscoveryResult:
    if section.AdminStatus != 'disabled':
        yield Service()
    elif params['admin_disabled']:
        yield Service()


def check_ospfv3_general(params, section: OspfV3General) -> CheckResult:
    yield Result(state=State.OK, summary=f'Router ID: {section.RouterId}')
    yield Result(state=State.OK, summary=f'ABR: {section.AreaBdrRtrStatus}')
    yield Result(state=State.OK, summary=f'ASBR: {section.ASBdrRtrStatus}')

    yield Result(state=State.OK, notice=f'Version number: {section.VersionNumber}')
    yield Result(state=State.OK, notice=f'AS scope LSA checksum: {section.AsScopeLsaCksumSum}')
    yield Result(state=State.OK, notice=f'max # of non-default AS-external-LSAs: {section.ExtAreaLsdbLimit}')
    yield Result(state=State.OK, notice=f'Reference bandwidth: {section.ReferenceBandwidth} KBPS')
    yield Result(state=State.OK, notice=f'Stub router support: {section.StubRouterSupport}')
    yield Result(state=State.OK, notice=f'Stub router advertisement: {section.StubRouterAdvertisement}')

    if section.AdminStatus == 'disabled':
        yield Result(state=State(params['state_admin_disabled']), notice='Admin disabled')

    yield Metric(name='ospfv3_general_lsacount', value=section.AsScopeLsaCount)
    yield Metric(name='ospfv3_general_rxnewlsas', value=section.RxNewLsas)
    yield Metric(name='ospfv3_general_extlsacount', value=section.ExtLsaCount)


register.snmp_section(
    name='ospfv3_general',
    parse_function=parse_ospfv3_general,
    fetch=SNMPTree(
        base='.1.3.6.1.2.1.191.1.1',  # OSPFV3-MIB::ospfv3AreaEntry
        oids=[
            OIDEnd(),  # ospfv3AreaId
            '1',  # ospfv3RouterId
            '2',  # ospfv3AdminStatus
            '3',  # ospfv3VersionNumber
            '4',  # ospfv3AreaBdrRtrStatus
            '5',  # ospfv3ASBdrRtrStatus
            '6',  # ospfv3AsScopeLsaCount
            '7',  # ospfv3AsScopeLsaCksumSum
            '8',  # ospfv3OriginateNewLsas
            '9',  # ospfv3RxNewLsas
            '10',  # ospfv3ExtLsaCount
            '11',  # ospfv3ExtAreaLsdbLimit
            '14',  # ospfv3ReferenceBandwidth
            '22',  # ospfv3StubRouterSupport
            '23',  # ospfv3StubRouterAdvertisement
        ]
    ),
    detect=exists('.1.3.6.1.2.1.191.1.1.1.*')  # OSPFV3-MIB::ospfv3RouterId
)

register.check_plugin(
    name='ospfv3_general',
    service_name='OSPFv3 general',
    discovery_function=discovery_ospfv3_general,
    check_function=check_ospfv3_general,
    check_default_parameters={
        'state_admin_disabled': 1,
    },
    check_ruleset_name='ospfv3_general',
    discovery_ruleset_name='discovery_ospfv3_general',
    discovery_default_parameters={
        'admin_disabled': False,
    },
)
