#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-10-31
#
# Monitor status of OSPFv3 neighbors
#
# 2021-09-17: rewritten for CMK 2.0
# 2021-10-02: added WATO options
#
# .1.3.6.1.2.1.191.1.9.1.4.1.0.50529054 = INTEGER: 2
# .1.3.6.1.2.1.191.1.9.1.5.1.0.50529054 = Hex-STRING: FE 80 00 00 00 00 00 00 01 92 01 68 00 00 00 03
# .1.3.6.1.2.1.191.1.9.1.6.1.0.50529054 = INTEGER: 19
# .1.3.6.1.2.1.191.1.9.1.7.1.0.50529054 = INTEGER: 1
# .1.3.6.1.2.1.191.1.9.1.8.1.0.50529054 = INTEGER: 8
# .1.3.6.1.2.1.191.1.9.1.9.1.0.50529054 = Counter32: 6
# .1.3.6.1.2.1.191.1.9.1.10.1.0.50529054 = Gauge32: 0
# .1.3.6.1.2.1.191.1.9.1.11.1.0.50529054 = INTEGER: 2
# .1.3.6.1.2.1.191.1.9.1.12.1.0.50529054 = INTEGER: 6
# .1.3.6.1.2.1.191.1.9.1.13.1.0.50529054 = INTEGER: 1
# .1.3.6.1.2.1.191.1.9.1.14.1.0.50529054 = Gauge32: 0
# .1.3.6.1.2.1.191.1.9.1.15.1.0.50529054 = INTEGER: 1
#
# OSPFV3-MIB::ospfv3NbrAddressType.1.0.50529054 = INTEGER: ipv6(2)
# OSPFV3-MIB::ospfv3NbrAddress.1.0.50529054 = Hex-STRING: FE 80 00 00 00 00 00 00 01 92 01 68 00 00 00 03
# OSPFV3-MIB::ospfv3NbrOptions.1.0.50529054 = INTEGER: 19
# OSPFV3-MIB::ospfv3NbrPriority.1.0.50529054 = INTEGER: 1
# OSPFV3-MIB::ospfv3NbrState.1.0.50529054 = INTEGER: full(8)
# OSPFV3-MIB::ospfv3NbrEvents.1.0.50529054 = Counter32: 6
# OSPFV3-MIB::ospfv3NbrLsRetransQLen.1.0.50529054 = Gauge32: 0
# OSPFV3-MIB::ospfv3NbrHelloSuppressed.1.0.50529054 = INTEGER: false(2)
# OSPFV3-MIB::ospfv3NbrIfId.1.0.50529054 = INTEGER: 6
# OSPFV3-MIB::ospfv3NbrRestartHelperStatus.1.0.50529054 = INTEGER: notHelping(1)
# OSPFV3-MIB::ospfv3NbrRestartHelperAge.1.0.50529054 = Gauge32: 0 seconds
# OSPFV3-MIB::ospfv3NbrRestartHelperExitReason.1.0.50529054 = INTEGER: none(1)
#
#
# sample string_table
# [
#  [
#   ['1.0.50529054', '2', '\xfe\x80\x00\x00\x00\x00\x00\x00\x01\x92\x01h\x00\x00\x00\x03',
#    '19', '1', '8', '6', '0', '2', '6', '1', '0', '1'],
#   ['10108.0.16843038', '2', '\xfe\x80\x00\x00\x00\x00\x00\x00\x01\x92\x01h\x00\x00\x00\x01',
#    '19', '1', '8', '6', '0', '2', '474', '1', '0', '1']
#  ],
#  [
#   ['1', 'Vlan1'],
#   ['5001', 'Port-channel1'],
#   ['10101', 'GigabitEthernet0/1'],
#   ['10102', 'GigabitEthernet0/2'],
#   ['10103', 'GigabitEthernet0/3'],
#   ['10104', 'GigabitEthernet0/4'],
#   ['10105', 'GigabitEthernet0/5'],
#   ['10106', 'GigabitEthernet0/6'],
#   ['10107', 'GigabitEthernet0/7'],
#   ['10108', 'GigabitEthernet0/8'],
#   ['10109', 'GigabitEthernet0/9'],
#   ['10110', 'GigabitEthernet0/10'],
#   ['10501', 'Null0'],
#   ['20567', 'Loopback0'],
#   ['20568', 'Loopback10'],
#   ['20569', 'Loopback20'],
#   ['20570', 'Loopback30']
#  ]
# ]

#

from dataclasses import dataclass
from typing import Dict, List

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    State,
    SNMPTree,
    exists,
    Metric,
    OIDEnd,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.utils.ospfv3 import (
    ospf_nbr_hellosuppressed,
    ospf_nbr_helperstatus,
    ospf_nbr_helperexitreason,
    render_ipv6_address,
    render_ipv4_neighbor_id,
    get_short_if_name,
    ospf_nbr_state,
)


@dataclass
class OspfV3Neighbor:
    nbrRtrId: str
    nbrOptions: str
    nbrPriority: int
    nbrState: int
    nbrEvents: int
    nbrLsRetransQLen: int
    nbrHelloSuppressed: str
    nbrRestartHelperStatus: str
    nbrRestartHelperAge: int
    nbrRestartHelperExitReason: str


def parse_ospfv3_neighbor(string_table: List[StringTable]) -> Dict[str, OspfV3Neighbor]:
    neighbors = {}
    neighborinfo, interfaces = string_table

    for neighbor in neighborinfo:
        OID_END, nbrAddressType, nbrAddress, nbrOptions, nbrPriority, nbrState, nbrEvents, nbrLsRetransQLen, \
        nbrHelloSuppressed, nbrRestartHelperStatus, nbrRestartHelperAge, nbrRestartHelperExitReason = neighbor

        if nbrAddressType == '2':  # IPv6 address
            nbrAddress = render_ipv6_address(nbrAddress)
            nbrIfIndex, nbrIfInstId, nbrRtrId = OID_END.split('.')

            nbrLocalInterface = ''
            for interface in interfaces:
                ifIndex, ifName = interface
                if ifIndex == nbrIfIndex:
                    nbrLocalInterface = get_short_if_name(ifName)

            neighbors[f'{nbrAddress} on {nbrLocalInterface}'] = OspfV3Neighbor(
                nbrRtrId=render_ipv4_neighbor_id(nbrRtrId),
                nbrOptions=nbrOptions,
                nbrPriority=int(nbrPriority),
                nbrState=int(nbrState),
                nbrEvents=int(nbrEvents),
                nbrLsRetransQLen=int(nbrLsRetransQLen),
                nbrHelloSuppressed=ospf_nbr_hellosuppressed(nbrHelloSuppressed),
                nbrRestartHelperStatus=ospf_nbr_helperstatus(nbrRestartHelperStatus),
                nbrRestartHelperAge=int(nbrRestartHelperAge),
                nbrRestartHelperExitReason=ospf_nbr_helperexitreason(nbrRestartHelperExitReason),
            )
    if neighbors:
        return neighbors


def discovery_ospfv3_neighbor(section: Dict[str, OspfV3Neighbor]) -> DiscoveryResult:
    for neighbor in section.keys():
        yield Service(item=neighbor)


def check_ospfv3_neighbor(item, params, section: Dict[str, OspfV3Neighbor]) -> CheckResult:

    not_found_state = params['state_not_found']

    for neighbour, neighbourAlias, neighbourNotFoundState in params.get('peer_list', []):
        if item == neighbour:
            yield Result(state=State.OK, summary=f'[{neighbourAlias}]')
            not_found_state = neighbourNotFoundState

    try:
        neighbor = section[item]
    except KeyError:
        yield Result(state=State(not_found_state), notice='Item not found in SNMP data')
        return

    # default monitoring states for ospfNbrState
    neighbor_states = {
        1: 2,  # down
        2: 1,  # attempt
        3: 1,  # init
        4: 0,  # twoWay
        5: 1,  # exchangeStart
        6: 1,  # exchange
        7: 1,  # loading
        8: 0,  # full
    }

    neighbor_states.update(params.get('neighbor_states', neighbor_states))  # update neighbor_states with params

    yield Result(state=State.OK, summary=f'Neighbor ID: {neighbor.nbrRtrId}')
    # yield Result(state=State.OK, summary=f'On interface: {neighbor.nbrLocalInterface}')

    yield Result(
        state=State(neighbor_states.get(neighbor.nbrState, 3)),
        summary=f'Status: {ospf_nbr_state(neighbor.nbrState)}'
    )

    yield Result(state=State.OK, notice=f'Neighbor options: {neighbor.nbrOptions}')
    yield Result(state=State.OK, notice=f'Neighbor priority: {neighbor.nbrPriority}')
    yield Result(state=State.OK, notice=f'Neighbor hello suppressed: {neighbor.nbrHelloSuppressed}')
    yield Result(state=State.OK, notice=f'Neighbor helper status: {neighbor.nbrRestartHelperStatus}')
    yield Result(state=State.OK, notice=f'Neighbor helper age: {neighbor.nbrRestartHelperAge}')
    yield Result(state=State.OK, notice=f'Neighbor helper exit reason: {neighbor.nbrRestartHelperExitReason}')

    yield Metric(name='ospfv3_events', value=neighbor.nbrEvents)
    yield Metric(name='ospfv3_lsretransqlen', value=neighbor.nbrLsRetransQLen)


register.snmp_section(
    name='ospfv3_neighbor',
    parse_function=parse_ospfv3_neighbor,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.2.1.191.1.9.1',  # OSPFV3-MIB::ospfv3NbrEntry
            oids=[
                OIDEnd(),  # nbrIfIndex, nbrIfInstId, nbrRtrId
                '4',  # 'ospfv3NbrAddressType'
                '5',  # 'ospfv3NbrAddress'
                '6',  # 'ospfv3NbrOptions'
                '7',  # 'ospfv3NbrPriority'
                '8',  # 'ospfv3NbrState
                '9',  # 'ospfv3NbrEvents'
                '10',  # 'ospfv3NbrLsRetransQLen'
                '11',  # 'ospfv3NbrHelloSuppressed'
                '13',  # 'ospfv3NbrRestartHelperStatus'
                '14',  # 'ospfv3NbrRestartHelperAge'
                '15',  # 'ospfv3NbrRestartHelperExitReason'
                # '12',  # 'ospfv3NbrIfId'
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.2.1.2.2.1',  # 
            oids=[
                '1',  # ifIndex
                '2',  # ifDescr
            ]
        ),
    ],
    detect=exists('.1.3.6.1.2.1.191.1.9.1.4.*')  # OSPFV3-MIB::ospfv3NbrAddressType
)

register.check_plugin(
    name='ospfv3_neighbor',
    service_name='OSPFv3 neighbor %s',
    discovery_function=discovery_ospfv3_neighbor,
    check_function=check_ospfv3_neighbor,
    check_default_parameters={
        'state_not_found': 3,
    },
    check_ruleset_name='ospfv3_neighbor',
)
