#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2019-01-11
#
# Monitor status of OSPFv3 virtual-links
#
# 2021-09-17: rewritten for CMK 2.0
# 2021-10-02: added WATO options
#
# .1.3.6.1.2.1.191.1.11.1.3.3.50529054 = INTEGER: 0
# .1.3.6.1.2.1.191.1.11.1.4.3.50529054 = Gauge32: 0
# .1.3.6.1.2.1.191.1.11.1.5.3.50529054 = INTEGER: 2
# .1.3.6.1.2.1.191.1.11.1.6.3.50529054 = Hex-STRING: 30 00 00 00 00 00 00 00 00 03 00 03 00 03 00 30
# .1.3.6.1.2.1.191.1.11.1.7.3.50529054 = INTEGER: 51
# .1.3.6.1.2.1.191.1.11.1.8.3.50529054 = INTEGER: 8
# .1.3.6.1.2.1.191.1.11.1.9.3.50529054 = Counter32: 6
# .1.3.6.1.2.1.191.1.11.1.10.3.50529054 = Gauge32: 1
# .1.3.6.1.2.1.191.1.11.1.11.3.50529054 = INTEGER: 1
# .1.3.6.1.2.1.191.1.11.1.12.3.50529054 = INTEGER: 26
# .1.3.6.1.2.1.191.1.11.1.13.3.50529054 = INTEGER: 1
# .1.3.6.1.2.1.191.1.11.1.14.3.50529054 = Gauge32: 0
# .1.3.6.1.2.1.191.1.11.1.15.3.50529054 = INTEGER: 1
#
# OSPFV3-MIB::ospfv3VirtNbrIfIndex.3.50529054 = INTEGER: 0
# OSPFV3-MIB::ospfv3VirtNbrIfInstId.3.50529054 = Gauge32: 0
# OSPFV3-MIB::ospfv3VirtNbrAddressType.3.50529054 = INTEGER: ipv6(2)
# OSPFV3-MIB::ospfv3VirtNbrAddress.3.50529054 = Hex-STRING: 30 00 00 00 00 00 00 00 00 03 00 03 00 03 00 30
# OSPFV3-MIB::ospfv3VirtNbrOptions.3.50529054 = INTEGER: 51
# OSPFV3-MIB::ospfv3VirtNbrState.3.50529054 = INTEGER: full(8)
# OSPFV3-MIB::ospfv3VirtNbrEvents.3.50529054 = Counter32: 6
# OSPFV3-MIB::ospfv3VirtNbrLsRetransQLen.3.50529054 = Gauge32: 0
# OSPFV3-MIB::ospfv3VirtNbrHelloSuppressed.3.50529054 = INTEGER: true(1)
# OSPFV3-MIB::ospfv3VirtNbrIfId.3.50529054 = INTEGER: 26
# OSPFV3-MIB::ospfv3VirtNbrRestartHelperStatus.3.50529054 = INTEGER: notHelping(1)
# OSPFV3-MIB::ospfv3VirtNbrRestartHelperAge.3.50529054 = Gauge32: 0 seconds
# OSPFV3-MIB::ospfv3VirtNbrRestartHelperExitReason.3.50529054 = INTEGER: none(1)
#
# sample string_table
# [
#  [
#   ['3.50529054', '0', '0', '2', '0\x00\x00\x00\x00\x00\x00\x00\x00\x03\x00\x03\x00\x03\x000',
#    '51', '8', '6', '0', '1', '26', '1', '0', '1']
#  ],
#  [
#   ['1', 'Vlan1'],
#   ['5001', 'Port-channel1'],
#   ['10101', 'GigabitEthernet0/1'],
#   ['10102', 'GigabitEthernet0/2'],
#   ['10103', 'GigabitEthernet0/3'],
#   ['10104', 'GigabitEthernet0/4'],
#   ['10105', 'GigabitEthernet0/5'],
#   ['10106', 'GigabitEthernet0/6'],
#   ['10107', 'GigabitEthernet0/7'],
#   ['10108', 'GigabitEthernet0/8'],
#   ['10109', 'GigabitEthernet0/9'],
#   ['10110', 'GigabitEthernet0/10'],
#   ['10501', 'Null0'],
#   ['20567', 'Loopback0'],
#   ['20569', 'Loopback10'],
#   ['20570', 'Loopback20'],
#   ['20571', 'Loopback30']
#  ]
# ]
#

from dataclasses import dataclass
from typing import Dict, List

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    Result,
    State,
    SNMPTree,
    exists,
    Metric,
    OIDEnd,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)

from cmk.base.plugins.agent_based.utils.ospfv3 import (
    ospf_nbr_hellosuppressed,
    ospf_nbr_helperstatus,
    ospf_nbr_helperexitreason,
    render_ipv6_address,
    render_ipv4_neighbor_id,
    get_short_if_name,
    ospf_nbr_state,
)


@dataclass
class OspfV3VirtualLink:
    VirtNbrId: str
    VirtNbrArea: int
    VirtNbrOptions: str
    VirtNbrState: int
    VirtNbrEvents: int
    VirtNbrLsRetransQLen: int
    VirtNbrHelloSuppressed: str
    VirtNbrRestartHelperStatus: str
    VirtNbrRestartHelperAge: int
    VirtNbrRestartHelperExitReason: str
    VitrNbrLocalInterface: str


def parse_ospfv3_virtuallink(string_table: List[StringTable]) -> Dict[str, OspfV3VirtualLink]:
    virtual_links = {}
    virtual_links_info, interfaces = string_table

    for virtual_link in virtual_links_info:

        OID_END, VirtNbrIfIndex, VirtNbrIfInstId, VirtNbrAddressType, VirtNbrAddress, VirtNbrOptions, VirtNbrState, \
        VirtNbrEvents, VirtNbrLsRetransQLen, VirtNbrHelloSuppressed, VirtNbrIfId, VirtNbrRestartHelperStatus, \
        VirtNbrRestartHelperAge, VirtNbrRestartHelperExitReason, = virtual_link

        if VirtNbrAddressType == '2':  # IPv6 address
            VirtNbrAddress = render_ipv6_address(VirtNbrAddress)
            VirtNbrArea, VirtNbrId = OID_END.split('.')

            nbrLocalInterface = ''
            for interface in interfaces:
                ifIndex, ifName = interface
                if ifIndex == VirtNbrIfIndex:
                    nbrLocalInterface = get_short_if_name(ifName)

            # virtual_links[f'{VirtNbrAddress} on {nbrLocalInterface}']= OspfV3VirtualLink(
            virtual_links[VirtNbrAddress] = OspfV3VirtualLink(
                VirtNbrId=render_ipv4_neighbor_id(VirtNbrId),
                VirtNbrArea=int(VirtNbrArea),
                VirtNbrOptions=VirtNbrOptions,
                VirtNbrState=int(VirtNbrState),
                VirtNbrEvents=int(VirtNbrEvents),
                VirtNbrLsRetransQLen=int(VirtNbrLsRetransQLen),
                VirtNbrHelloSuppressed=ospf_nbr_hellosuppressed(VirtNbrHelloSuppressed),
                VirtNbrRestartHelperStatus=ospf_nbr_helperstatus(VirtNbrRestartHelperStatus),
                VirtNbrRestartHelperAge=int(VirtNbrRestartHelperAge),
                VirtNbrRestartHelperExitReason=ospf_nbr_helperexitreason(VirtNbrRestartHelperExitReason),
                VitrNbrLocalInterface=nbrLocalInterface,
            )
    if virtual_links:
        return virtual_links


def discovery_ospfv3_virtuallink(section: Dict[str, OspfV3VirtualLink]) -> DiscoveryResult:
    for virtual_link in section.keys():
        yield Service(item=virtual_link)


def check_ospfv3_virtuallink(item, params, section: Dict[str, OspfV3VirtualLink]) -> CheckResult:

    not_found_state = params['state_not_found']

    for virtual_link, alias, NotFoundState in params.get('peer_list', []):
        if item == virtual_link:
            yield Result(state=State.OK, summary=f'[{alias}]')
            not_found_state = NotFoundState

    try:
        virtual_link = section[item]
    except KeyError:
        yield Result(state=State(not_found_state), notice='Item not found in SNMP data')
        return

    # default monitoring states for ospfNbrState
    neighbor_states = {
        1: 2,  # down
        2: 1,  # attempt
        3: 1,  # init
        4: 0,  # twoWay
        5: 1,  # exchangeStart
        6: 1,  # exchange
        7: 1,  # loading
        8: 0,  # full
    }

    neighbor_states.update(params.get('neighbor_states', neighbor_states))  # update neighbor_states with params

    yield Result(state=State.OK, summary=f'Neighbor ID: {virtual_link.VirtNbrId}')
    yield Result(state=State.OK, summary=f'Area: {virtual_link.VirtNbrArea}')

    yield Result(
        state=State(neighbor_states.get(virtual_link.VirtNbrState, 3)),
        summary=f'Status: {ospf_nbr_state(virtual_link.VirtNbrState)}'
    )

    yield Result(state=State.OK, notice=f'Virtual link options: {virtual_link.VirtNbrOptions}')
    yield Result(state=State.OK, notice=f'Virtual link hello suppressed: {virtual_link.VirtNbrHelloSuppressed}')
    yield Result(state=State.OK, notice=f'Virtual link helper status: {virtual_link.VirtNbrRestartHelperStatus}')
    yield Result(state=State.OK, notice=f'Virtual link helper age: {virtual_link.VirtNbrRestartHelperAge}')
    yield Result(state=State.OK,
                 notice=f'Virtual link helper exit reason: {virtual_link.VirtNbrRestartHelperExitReason}')
    yield Result(state=State.OK, notice=f'Virtual link local interface: {virtual_link.VitrNbrLocalInterface}')

    yield Metric(name='ospfv3_events', value=virtual_link.VirtNbrEvents)
    yield Metric(name='ospfv3_lsretransqlen', value=virtual_link.VirtNbrLsRetransQLen)


register.snmp_section(
    name='ospfv3_virtuallink',
    parse_function=parse_ospfv3_virtuallink,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.2.1.191.1.11.1',  # OSPFV3-MIB::ospfv3IfTable
            oids=[
                OIDEnd(),  # ospfv3IfIndex, ospfv3IfInstId
                '3',  # ospfv3VirtNbrIfIndex
                '4',  # ospfv3VirtNbrIfInstId
                '5',  # ospfv3VirtNbrAddressType
                '6',  # ospfv3VirtNbrAddress
                '7',  # ospfv3VirtNbrOptions
                '8',  # ospfv3VirtNbrState
                '9',  # ospfv3VirtNbrEvents
                '10',  # ospfv3VirtNbrLsRetransQLen
                '11',  # ospfv3VirtNbrHelloSuppressed
                '12',  # ospfv3VirtNbrIfId
                '13',  # ospfv3VirtNbrRestartHelperStatus
                '14',  # ospfv3VirtNbrRestartHelperAge
                '15',  # ospfv3VirtNbrRestartHelperExitReason
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.2.1.2.2.1',  # 
            oids=[
                '1',  # ifIndex
                '2',  # ifDescr
            ]
        ),
    ],
    detect=exists('.1.3.6.1.2.1.191.1.11.1.3.*')  # OSPFV3-MIB::ospfv3IfAreaId
)

register.check_plugin(
    name='ospfv3_virtuallink',
    service_name='OSPFv3 virtual-link %s',
    discovery_function=discovery_ospfv3_virtuallink,
    check_function=check_ospfv3_virtuallink,
    check_default_parameters={
        'state_not_found': 3,
    },
    check_ruleset_name='ospfv3_virtuallink',
)
