#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2021-09-17
#
# include file, will be used with ospfv3_area, ospfv3_general, ospfv3_interface, ospfv3_neighbor, ospfv3_virtuallink
#
import re


def get_area_type(area_import_as_extern: str, area_summary: str) -> str:
    areatype = 'UNKNOWN'
    if int(area_import_as_extern) == 1 and int(area_summary) == 2:
        areatype = 'normal'
    if int(area_import_as_extern) == 2 and int(area_summary) == 1:
        areatype = 'totaly subby'
    if int(area_import_as_extern) == 2 and int(area_summary) == 2:
        areatype = 'stubby'
    if int(area_import_as_extern) == 3 and int(area_summary) == 2:
        areatype = 'NSSA'
    if int(area_import_as_extern) == 3 and int(area_summary) == 1:
        areatype = 'totaly NSSA'
    return areatype


def render_ipv4_neighbor_id(router_id: str) -> str:
    """
    >>> from pprint import pprint
    >>> pprint(render_ipv4_neighbor_id('50529054'))
    3.3.3.30
    """
    router_id = int(router_id)
    firstoctet = router_id // (256 * 256 * 256)
    router_id = router_id - (firstoctet * 256 * 256 * 256)
    secondocted = router_id // (256 * 256)
    router_id = router_id - (secondocted * 256 * 256)
    thirdorted = router_id // 256
    router_id = router_id - (thirdorted * 256)
    router_id = f'{firstoctet}.{secondocted}.{thirdorted}.{router_id}'
    return router_id


_interface_displayhints = {
    'ethernet': 'eth',
    'fastethernet': 'Fa',
    'gigabitethernet': 'Gi',
    'tengigabitethernet': 'Te',
    'fortygigabitethernet': 'Fo',
    'hundredgigabitethernet': 'Hu',
    'port-channel': 'Po',
    'tunnel': 'Tu',
    'loopback': 'Lo',
    'cellular': 'Cel',
    'vlan': 'Vlan',
    'management': 'Ma',
}


def get_short_if_name(ifname: str) -> str:
    """
    returns short interface name from long interface name
    ifname: is the long interface name
    :type ifname: str
    """

    for ifname_prefix in _interface_displayhints.keys():
        if ifname.lower().startswith(ifname_prefix.lower()):
            ifname_short = _interface_displayhints[ifname_prefix]
            return ifname.lower().replace(ifname_prefix.lower(), ifname_short, 1)
    return ifname


def _shorten_ipv6_adress(address: str) -> str:
    """
    >>> from pprint import pprint
    >>> pprint(_shorten_ipv6_adress('20:03:00:00:00:00:00:00:01:92:01:68:00:00:00:01'))
    2003::192:168:0:1
    """
    # '20:03:00:00:00:00:00:00:01:92:01:68:00:00:00:01'
    address = address.split(':')
    span = 2
    address = [''.join(address[i:i + span]) for i in range(0, len(address), span)]
    # ['2003', '0000', '0000', '0000', '0192', '0168', '0000', '0001']
    for m in range(0, len(address)):
        address[m] = re.sub(r'^0{1,3}', r'', address[m])
    # ['2003', '0', '0', '0', '192', '168', '0', '1']
    address = ':'.join(address)
    # '2003:0:0:0:192:168:0:1'
    zeros = ':0:0:0:0:0:0:'
    while not zeros == '':
        if zeros in address:
            address = address.replace(zeros, '::', 1)
            zeros = ''
        else:
            zeros = zeros[:-2]
    if address == '0::0':
        address = '::'
    # '2003::192:168:0:1'
    return address


def render_ipv6_address(bytestring) -> str:
    address = ':'.join([f'{hex(ord(m))[2:]:0>2}' for m in bytestring]).replace(' ', '0').upper()
    address = _shorten_ipv6_adress(address)

    return address


def ospf_stub_router_advertisement(st: str) -> str:
    names = {
        '1': 'do Not Advertise',
        '2': 'advertise',
    }
    return names.get(st, st)


def ospf_exit_reason(st: str) -> str:
    names = {
        '1': 'none',
        '2': 'in Progress',
        '3': 'completed',
        '4': 'timed Out',
        '5': 'topology Changed',
    }
    return names.get(st, st)


def ospf_restart_status(st: str) -> str:
    names = {
        '1': 'not Restarting',
        '2': 'planned Restart',
        '3': 'unplanned Restart',
    }
    return names.get(st, st)


def ospf_restart_support(st: str) -> str:
    names = {
        '1': 'none',
        '2': 'planned Only',
        '3': 'planned And Unplanned',
    }
    return names.get(st, st)


def ospf_admin_status(st: str) -> str:
    names = {
        '1': 'enabled',
        '2': 'disabled',
    }
    return names.get(st, st)


def ospf_abr_asbr_status(st: str) -> str:
    names = {
        '1': 'yes',
        '2': 'no',
    }
    return names.get(st, st)


def ospf_stub_router_support(st: str) -> str:
    names = {
        '1': 'yes',
        '2': 'no',
    }
    return names.get(st, st)


def ospf_area_impotasextern(st: str) -> str:
    names = {
        '1': 'import External',
        '2': 'import no External',
        '3': 'import Nssa',
    }
    return names.get(st, st)


def ospf_area_summary(st: str) -> str:
    names = {
        '1': 'no Area Summary',
        '2': 'send Area Summary',
    }
    return names.get(st, st)


def ospf_area_translatorrole(st: str) -> str:
    names = {
        '1': 'always',
        '2': 'candidate',
    }
    return names.get(st, st)


def ospf_area_translatorrolestate(st: str) -> str:
    names = {
        '1': 'enabled',
        '2': 'elected',
        '3': 'disabled',
    }
    return names.get(st, st)


def ospf_area_submetrictype(st: str) -> str:
    names = {
        '1': 'ospfv3 Metric',
        '2': 'comparable Cost',
        '3': 'non Comparable',
    }
    return names.get(st, st)


def ospf_area_teeenabled(st: str) -> str:
    names = {
        '1': 'enabled',
    }
    return names.get(st, 'disabled')


def ospf_if_type(st: str) -> str:
    names = {
        '1': 'broadcast',
        '2': 'nbma',
        '3': 'pointToPoint',
        '5': 'pointToMultipoint',
    }
    return names.get(st, st)


def ospf_if_admin_status(st: str) -> str:
    names = {
        '1': 'enabled',
        '2': 'disabled',
    }
    return names.get(st, st)


def ospf_if_state(st: str) -> str:
    names = {
        '1': 'down',
        '2': 'loopback',
        '3': 'waiting',
        '4': 'P2P',
        '5': 'DR',
        '6': 'BDR',
        '7': 'other DR',
        '8': 'standby',
    }
    return names.get(st, st)


def ospf_nbr_state(st: int) -> str:
    names = {
        1: 'down',
        2: 'attempt',
        3: 'init',
        4: 'twoWay',
        5: 'exchangeStart',
        6: 'exchange',
        7: 'loading',
        8: 'full',
    }
    return names.get(st, st)


def ospf_nbr_addresstype(st: str) -> str:
    names = {
        '0': 'unknown',
        '1': 'ipv4',
        '2': 'ipv6',
        '3': 'ipv4z',
        '4': 'ipv6z',
        '16': 'dns',
    }
    return names.get(st, st)


def ospf_nbr_hellosuppressed(st: str) -> str:
    names = {
        '1': 'true',
        '2': 'false',
    }
    return names.get(st, st)


def ospf_nbr_helperstatus(st: str) -> str:
    names = {
        '1': 'not Helping',
        '2': 'helping',
    }
    return names.get(st, st)


def ospf_nbr_helperexitreason(st: str) -> str:
    names = {
        '1': 'none',
        '2': 'inProgress',
        '3': 'completed',
        '4': 'timedOut',
        '5': 'topologyChanged',
    }
    return names.get(st, st)
